#include <stdlib.h>
#include <stdio.h>
#include <GL/glut.h>
#include <math.h>

#include "tb.h"

#define M_BODY	1
#define M_TORSO	2
#define M_HEAD	3
#define M_VIEWPOINT	4
#define M_LIGHT 5

GLfloat lightpos[4] = { 10.0, 10.0, -30.0, 0.0 };
GLfloat lightamb[4] = { 0.7, 0.7, 0.7, 1.0 };
GLfloat lightdif[4] = { 0.5, 0.5, 0.5, 1.0 };
GLfloat lightspe[4] = { 0.5, 0.5, 0.5, 1.0 };

/* Parametres */
float squash[3];
float tilt[3];
float radius[3];

/* Pr interface utilisateur */
int beginx, beginy, width, height;
float beginSquash, beginTilt;
int objectMoving; /* quel morceau du bonhomme on bouge  
		     0 = le corps
		     1 = le torse
		     2 = la tete
		     3 = le point de vue, 
		     4 = la lampe */

void enBlanc()
{
    float ambiant[3] = {0.4, 0.4, 0.4};
    float diffuse[3] = {0.2, 0.2, 0.2};
    float specular[3] = {0.4, 0.4, 0.5};

    glMaterialfv(GL_FRONT, GL_AMBIENT, ambiant);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
    glMaterialf(GL_FRONT, GL_SHININESS, 1200);
}

void enNoir()
{
  float diffuse2[3] = {0.05, 0.05, 0.05};
  glMaterialfv(GL_FRONT, GL_AMBIENT, diffuse2);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse2);
  glMaterialfv(GL_FRONT, GL_SPECULAR, diffuse2);
  glMaterialf(GL_FRONT, GL_SHININESS, 100);
}

void enRouge()
{
  float diffuse3[3] = {0.5, 0.05, 0.05};
  glMaterialfv(GL_FRONT, GL_AMBIENT, diffuse3);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse3);
  glMaterialfv(GL_FRONT, GL_SPECULAR, diffuse3);
  glMaterialf(GL_FRONT, GL_SHININESS, 100);
}

void bonhommeDeNeige()
{
  glPushMatrix();
    glScalef(0.3, 0.3, 0.3);

    /* Le sol */
    glRotatef(-30, 1, 0, 0);
    glRotatef(-20, 0, 1, 0);
    glTranslatef(0.0, -1.0, 0.0);
    enBlanc();
    glBegin(GL_POLYGON);
    glVertex3f(-10, 0, -10);
    glVertex3f(-10, 0, 10);
    glVertex3f(10, 0, 10);
    glVertex3f(10, 0, -10);
    glEnd();
    /* Le corps */
    glTranslatef(0.0, radius[0], 0.0);
    glutSolidSphere(radius[0], 20, 20);
    /* C'est ici qu'il reste du travail...*/

  glPopMatrix();
}

void handle_main_menu(int item)
{
  switch(item) {
  case M_BODY:
    objectMoving = 0;
    glutPostRedisplay();
    break;
  case M_TORSO:
    objectMoving = 1;
    glutPostRedisplay();
    break;
  case M_HEAD:
    objectMoving = 2;
    break;
  case M_VIEWPOINT:
    /* do something else */
    break;
  }
}

void
mouse(int button, int state, int x, int y)
{
  if (objectMoving == 5) 
    tbMouse(button, state, x, y);
  else if (state == GLUT_DOWN) {
    beginx = x;
    beginy = y;
    beginSquash = squash[objectMoving];
    beginTilt = tilt[objectMoving];
  }
  glutPostRedisplay();
}

void
motion(int x, int y)
{
  if (objectMoving == 5)
    tbMotion(x, y);
  else {
    squash[objectMoving] = beginSquash+(y - beginy)/(float)height;
    tilt[objectMoving] = beginTilt + (100*(x - beginx))/(float)width;
  }
  glutPostRedisplay();
}

void parsekey(unsigned char key, int x, int y)
{
  /* Fonction utilisee pour le controle par l'utilisateur */
  switch (key)
    {
    case 27: exit(0);
    case ' ': 
      squash[0] = 1.0;
      squash[1] = 1.0;
      squash[2] = 1.0;
      tilt[0] = 0.0;
      tilt[1] = 0.0;
      tilt[2] = 0.0;
      break;
    case 'b': objectMoving = 0; break;
    case 't': objectMoving = 1; break;
    case 'h': objectMoving = 2; break;
    case 'l': objectMoving = 4; break;
    case 'v': objectMoving = 3; break;
    default:
      return;
    }
  glutPostRedisplay();
}

void parsekey_special(int key, int x, int y)
{
  int ob;
  if (objectMoving > 3) ob = 0; else ob = objectMoving;
  switch (key)
    {
    case GLUT_KEY_UP:	if (objectMoving < 3) { squash[ob] += 0.05; break; }
    case GLUT_KEY_DOWN:	if (objectMoving < 3) { squash[ob] -= 0.05; break; }
    case GLUT_KEY_RIGHT: if (objectMoving < 3) { tilt[ob] -= 5; break; }
    case GLUT_KEY_LEFT:	if (objectMoving < 3) { tilt[ob] += 5; break; }
    default:
      return;
    }
  glutPostRedisplay();
}

void myReshape(int w, int h)
{
  int xmin, ymin;
  tbReshape(w, h);
  xmin = 0;
  ymin = 0;
  if (w > h) {
    xmin = (w-h)/2;
    w = h;
  } else {
    ymin = (h-w)/2;
    h = w;
  }
  width = w;
  height = h;
  glViewport(xmin, ymin, w, h);
  glutPostRedisplay();
}

void display(void)
{
  glPushMatrix();
  tbMatrix();
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

  bonhommeDeNeige();

  glutSwapBuffers();
  glPopMatrix();
}

void myinit(void)
{
  /* Trackball */
  glDepthRange (0.1, 10);
  tbInit(GLUT_LEFT_BUTTON);
  tbAnimate(GL_TRUE);

  glShadeModel (GL_SMOOTH);
  glFrontFace(GL_CW);
  glEnable(GL_DEPTH_TEST);
  glCullFace(GL_BACK);
  glEnable(GL_CULL_FACE);
  
  glLoadIdentity();
  glClearColor(0.0, 0.0, 0.0, 1.0);
  glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
  glLightfv(GL_LIGHT0, GL_AMBIENT, lightamb);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, lightdif);
  glLightfv(GL_LIGHT0, GL_SPECULAR, lightspe);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glPolygonMode(GL_FRONT, GL_FILL);
  squash[0] = 1.0;
  squash[1] = 1.0;
  squash[2] = 1.0;
  tilt[0] = 0.0;
  tilt[1] = 0.0;
  tilt[2] = 0.0;
  radius[0] = 0.9;
  radius[1] = 0.7;
  radius[2] = 0.5;
  objectMoving = 5;
}

int main(int argc, char *argv[]) 
{
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE | GLUT_MULTISAMPLE |
		      GLUT_STENCIL);
  glutCreateWindow("TD1");
  glutDisplayFunc(display);
  glutInitWindowSize(1200, 1200);
  glutInitWindowPosition(200, 0);
  glutMouseFunc(mouse);
  glutMotionFunc(motion);
  glutKeyboardFunc(parsekey);
  glutSpecialFunc(parsekey_special);
  glutReshapeFunc(myReshape);
  myinit();
    
  glutCreateMenu(handle_main_menu);
  glutAddMenuEntry("Moving the body", M_BODY);
  glutAddMenuEntry("Moving the torso", M_TORSO);
  glutAddMenuEntry("Moving the head", M_HEAD);
  glutAddMenuEntry("-----------------", -1);
  glutAddMenuEntry("Moving the viewpoint", M_VIEWPOINT);
  glutAddMenuEntry("Moving the light", M_LIGHT);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
  glutSwapBuffers();
  glutMainLoop();
  return 0;
}
